@@ -4,12 +4,12 @@ use File::Path 'mkpath';
 use File::Copy 'copy';
 use Config;
 
-my $VERSION = '1.01';			# Changelog at end
+my $VERSION = '1.02';			# Changelog at end
 die "Debugging cycle detected"		# set to -1 to allow extra iteration
   if ++$ENV{PERL_DEBUG_MCODE_CYCLE} > 1;
 
 my %opt;
-$opt{$1} = shift while ($ARGV[0] || 0) =~ /^-([dq1O])$/;
+$opt{$1} = shift while ($ARGV[0] || 0) =~ /^-([dq1OB])$/;
 if ($opt{1}) {
   open STDERR, '>&STDOUT' or warn "can't redirect STDERR to STDOUT";
 } else {
@@ -20,7 +20,7 @@ my $bd = (my $bd0 = 'dbg-bld') . ($opt{O} || '');
 @ARGV >= 1 or die <<EOP;
 
 Usage:
- $0 [-d] [-q] [-1] [-O] check-module [failing-script1 failing-script2 ...]
+ $0 [-B] [-d] [-q] [-1] [-O] check-module [failing-script1 failing-script2 ...]
 
 A tool to simplify remote debugging of build problems for XSUB modules.
 By default, output goes to STDERR (to pass through the test suite wrappers).
@@ -44,6 +44,7 @@ Options:	With -d, prefers dbx to gdb (DEFAULT: prefer gdb).
 			failure of loading is detected.
 		With -1, all output goes to STDOUT.
 		With -O, makes a non-debugging build.
+		With -B, builds in a subdirectory even if no debugger was found.
 
 Assumptions:
   Should be run in the root of a distribution, or its immediate subdir.
@@ -130,12 +131,32 @@ exit 0 unless @ARGV or not $chk_module;
 
 my $gdb = `gdb --version` unless $opt{d};
 my $dbx = `dbx -V -c quit` unless $gdb;
-$gdb = `gdb --version` unless $gdb or $dbx;
-die "Can't find gdb or dbx" unless defined $gdb or defined $dbx;
-die "Can't parse output of gdb --version"
-  unless $dbx or $gdb =~ /\b GDB \b | \b Copyright \b .* \b Free Software \b/x;
-die "Can't parse output of `dbx -V -c quit'"
-  unless $gdb or $dbx =~ /\b dbx \s+ debugger \b/xi;
+unless ($gdb or $dbx) {
+  unless ($gdb = `gdb --version`) {
+    my @cand;
+    for my $dir (split m(/), ($ENV{PATH} || '')) {
+      for my $f (<$dir/*>) {
+        push @cand, $f if $f =~ m{dbx|gdb|lldb}i and -x $f;
+      }
+    }
+    warn 'Possible candidates for debuggers: {{{'. join('}}} {{{', @cand), '}}}' if @cand;
+  };
+}
+
+sub report_no_debugger () {
+  die "Can't find gdb or dbx" unless defined $gdb or defined $dbx;
+  die "Can't parse output of gdb --version"
+    unless $dbx or $gdb =~ /\b GDB \b | \b Copyright \b .* \b Free Software \b/x;
+  die "Can't parse output of `dbx -V -c quit'"
+    unless $gdb or $dbx =~ /\b dbx \s+ debugger \b/xi;
+}
+
+my $postpone = not (defined $gdb or defined $dbx);
+if ($opt{B}) {
+  warn "No debugger found.  Nevertheless, I build a new version per -B switch." if $postpone;
+} else {
+  report_no_debugger();
+}
 
 die "Directory $bd exist; won't overwrite" if -d $bd;
 mkdir $bd or die "mkdir $bd: $!";
@@ -179,6 +200,8 @@ my $make = $Config{make};
 $make = 'make' unless defined $make;
 system $make and die "system($make): rc=$?";
 
+report_no_debugger();		# Reached without a debugger only with -B
+
 my $p = ($^X =~ m([\\/]) ? $^X : `which perl`) || $^X;
 chomp $p unless $p eq $^X;
 my(@cmd, $ver);
@@ -258,4 +281,6 @@ __END__
 1.01	Better docs.
 	Emit the child's command line before running.
 	On newer Perls %Config::Config may be read-only.
-	Move gdb report of /proc to the end (not working on cygwin 2016, gdb v7.8)
\ No newline at end of file
+	Move gdb report of /proc to the end (not working on cygwin 2016, gdb v7.8)
+1.02	Scan and report possible candidates for a debugger.
+	New option -B.
\ No newline at end of file
