@@ -17,6 +17,13 @@ my(@tests, %seen, $current_num, $userfun, $installed, $skip_gnuplot, %not_yet_de
 
 my $mess = '';
 $mess = CORE::shift @tests if @tests and $tests[0] !~ /^\?/; # Messages
+my @skip_fun;
+my $skip_fun_rx = qr/(?!)/;
+my $matched_par;
+$matched_par = qr[[^()]*(?:\((??{$matched_par})\)[^()]*)*];		# arbitrary string with ( and ) matching
+my %ourvars;
+my $ourvars_rx = qr/(?!)/;
+
 
 (my $ifile = $file) =~ s[(?<=/test/)(32|64)/][in/];
 {
@@ -25,14 +32,19 @@ $mess = CORE::shift @tests if @tests and $tests[0] !~ /^\?/; # Messages
   $intests = <FROM>;
   close FROM or die "close: $!";
 }
-(my $pref = $intests) =~ s/^(\\e|default\(echo,[^\n]*\n)(.*)//ms and $rest = $2;
+my $pref;
+($pref = $intests) =~ s/^(\\e|default\(echo,[^\n]*\n)(.*)//ms and CORE::length $pref and $rest = $2;
 
 $popped++, pop @tests if ($tests[-1] || 0) =~ /^\\q/;
 
+my $skip_eval = (Math::Pari::pari_version_exp() >= 2004002);
+my $use_dollars_in_argsign = $skip_eval;
+
 if ($pref =~ /^\s*\\\\\s*package:(.*)/m) {
   print "1..0 # skipped: package dependency: $1\n";
   @tests = ();
 } elsif (my $tests = @tests) {
+  $tests +=  !!$rest;
   print "1..$tests\n";
 } elsif ($rest) {
   print "1..1\nnot ok 1 # no tests found in `$file', but input has echo enabled\n";
@@ -42,7 +54,20 @@ if ($pref =~ /^\s*\\\\\s*package:(.*)/m) {
 }
 
 # prec($3 || $1, 1) if $mess =~ /.*realprecision = (\d+) significant digits( \((\d+) digits displayed\))?/; # take the latest one
-parse_as_gp($pref) if $rest;
+if ($rest) {	# 'define'
+  (my $Pref = $pref) =~ s/^(.*)$/### $1/mg ;
+  print("# The following `? ' are the output from parse_as_gp() for\n$Pref");
+  my $Err;
+  eval { parse_as_gp $pref, sub ($) {"main::".CORE::shift}, 'echo'; 1}
+    or $Err = $@;
+  $current_num++;
+  if (defined $Err) {  #  failed to process, but do emit the expected-for-test output
+    print "not ok $current_num # in no-echo group err=$Err\n";
+  } else {
+    print "ok $current_num # Skipping translation to Math::Pari (no-echo group done in PARI, see above)\n";
+  }
+}
+$rest ||= $intests;
 
 #Math::Pari::dumpStack,allocatemem(8e6),Math::Pari::dumpStack if $file =~ /intnum/; # Due to our allocation policy we need more?
 
@@ -91,10 +116,11 @@ if ($file =~ /plot|graph|all/) {
 my $started = 0;
 
 main_loop:
-for (@tests) {
+while (@tests) {
+  $_ = CORE::shift @tests;
   #print "Doing `$_'";
   1 while s/^\\\\.*\n//;	# Comment
-  my $bad = /^\\/;			# \precision =
+  my $bad = /^\\(?!p[bs]?\s*\d)/;		# \precision =
   my $wasbadprint = /\b(plot)\b/;
   my $wasprint = /\b((|p|tex)print(tex)?|plot)\b/;
   s/\s*\n\?\s*\Z// or not $popped or die "Not terminated: `$_'\n";
@@ -171,7 +197,7 @@ sub mformat {
   @_ = grep {!/^$/} @_;		# remove empty lines
   return join("\t", @_) if grep {!/^\s*\[.*\]\s*$/} @_;	# Not matrix
   #return join("\t", @_) if grep {!/^\s*\([^,]*,\s*$/} @_; # Extra commas
-  map {s/^\s*\[(.*)\]\s*$/$1/} @_;
+  map {s/^\s*\[\s*(.*?)\s*\]\s*$/$1/} @_;
   my @arr = map { join ', ', split } @_;
   '[' . join('; ', @arr) . ']';
 }
@@ -242,10 +268,34 @@ sub pre_update_seen ($) {
   eval "use vars '\$$sym'; 1" or die "use vars: $@";
 }
 
+sub subify_iterators ($$) {
+  my($pre, $code, $subargs, $subdecl) = (CORE::shift, CORE::shift, '', '');
+  if ($use_dollars_in_argsign) {
+    $subargs = ' ($) ';
+    if ($pre =~ /^(v?vector|fordiv|sumdiv|plothexport)\(/) {
+      $pre =~ /^\w+\s*\([^,]*,\s*([\$\w]+)\s*,/ or die "Unexpected iterator `$pre\{\{\{$code\}\}\}'";
+      $subdecl = "my $1 = CORE::shift;"
+    } else {
+      $pre =~ /^\w+\s*\(\s*([\$\w]+)\s*,/ or die "Unexpected iterator `$pre\{\{\{$code\}\}\}'";
+      $subdecl = "my $1 = CORE::shift;"
+    }
+  }
+#	      /$1 sub$subargs\{$2}/xg;
+  "$pre sub$subargs\{$subdecl$code\}";
+}
+
+sub filter_res ($) { # In PARIs Mod() output there is an extra space comparing to ours
+  my $r = CORE::shift;
+  return $r unless $r =~ /\bMod\(/;
+  $r =~ s/,\s+/,/g;
+  return $r;
+}
+
 my $prev;
 sub process_test {
   my ($in, $noans, $out) = @_;
-  my($res, $rres, $rout);
+#	warn("<<<$in>>>, $noans, <<<@$out>>>");
+  my($IN, $res, $rres, $rout) = $in;
   my $ini_time = time;
   my $doprint;
   $doprint = 1 if $noans eq 'print';
@@ -259,6 +309,7 @@ sub process_test {
   $in =~ s/\b(\w+)\s*!/ ifact($1)/g; # !
   $in =~ s/,\s*(?=,)/, \$DEFAULT /g;	# Default arguments?
   $in =~ s/^default\(realprecision,(.*)\)/\\p $1/; # Some cases of default()
+  $in =~ s/^default\(realbitprecision,(.*)\)/\\pb $1/; # Some cases of default()
   $in =~ s/^default\(seriesprecision,(.*)\)/\\ps $1/; # Some cases of default()
   $in =~ s/(\w+)\s*\\(\w+(\s*\^\s*\w+)?)/gdivent($1,$2)/g; # random\10^8
   $in =~ s/%(?!\s*[\[_\w])/\$was_prev/g; # foo(%)
@@ -280,16 +331,18 @@ sub process_test {
 	    ' $
 	  /deriv$1/x; # ((x+y)^5)'
   if ($in =~ /^\\p\s*(\d+)/) {
-    prec($1);
+    prec("$1");
+  } elsif ($in =~ /^\\pb\s*(\d+)/) {		# \\ for division unsupported
+    bprec("$1");
   } elsif ($in =~ /^\\ps\s*(\d+)/) {		# \\ for division unsupported
-    sprec($1);
+    sprec("$1");
   } elsif ($in =~ /\\/) {		# \\ for division unsupported
     $current_num--;
     process_error($in, $out, '\\');
   } elsif ($in =~ /^(\w+)\s*\([^()]*\)\s*=/ and 0) { # XXXX Not implemented yet
     $current_num--;
     process_definition($1, $in);
-  } elsif ($in =~ /[!\']/) {	# Factorial
+  } elsif ($in =~ /!(?![\(\w])|\'/) {	# Factorial (unless in !foo())
     print "# `$in'\nok $current_num # Skipping (ifact/deriv)\n";
   } else {
     # work with "^", need to treat differently inside o()
@@ -338,21 +391,22 @@ sub process_test {
 	      \)
 	    /process_cond($1, $2, $3, $4, $in)/xge; # if(a,b,c)
  };
+    my $RET;
     if ($in =~ /\[[^\]]*;/) {	# Matrix
       print "# `$in'\nok $current_num # Skipping (matrix notation)\n";
-      return;
+      $RET = 1;
     } elsif ($in =~ /Skip this `(.*)'/) {
       print "# `$1'\nok $current_num # Skipping (runaway conversion)\n";
-      return;
+      $RET = 1;
     } elsif ($in =~ /&for\s*\([^\)]*$/) {	# Special case
       print "# `$in'\nok $current_num # Skipping (runaway input line)\n";
-      return;
+      $RET = 1;
     } elsif ($in =~ /(^|[\(=,])%/) {
       print "# `$in'\nok $current_num # Skipping (history notation)\n";
-      return;
+      $RET = 1;
     } elsif ($in =~ /\b(get(heap|stack)|Total time spent.*gettime)\b/) {
       print "# `$in'\nok $current_num # Silently skipping: meaningless for Math::Pari\n";
-      return;
+      $RET = 1;
     } elsif ($in =~ /
 		      (
 			\b
@@ -362,7 +416,7 @@ sub process_test {
                         )
 			\b
 		      |
-			(\w+) \s* \( \s* \w+ \s* \) =
+			(\w+) \s* \( \s* (?: (?!\d)\w+ \s* (?:, \s* (?!\d)\w+ \s* )* )? \) \s* =(?!=)
 		      |
 			\b install \s* \( \s* (\w+) \s* , [^()]* \)
 		      |
@@ -377,11 +431,6 @@ sub process_test {
 		      )
 		    /x) {
       if (defined $3) {
-	if (defined $userfun) {
-	  $userfun .= "|$3";
-	} else {
-	  $userfun = $3;
-	}
 	print "# User function `$3'.\n";
       }
       if (defined $4) {
@@ -392,32 +441,99 @@ sub process_test {
 	}
 	print "# Installed function `$4'.\n";
       }
-      # It is not clear why changevar gives a different answer in GP
-      print "# `$in'\nok $current_num # Skipping (converting test for '$1' needs additional work)\n";
-      return;
+      if ($1 eq 'default' and $in =~ /^default\s*\(\s*echo\s*,\s*0\s*\)\s*(;\s*)?$/) {	# See the test 'ff' - but this is not present in the OUTPUT!!!???
+        if ($rest =~ s/\A.*?^default\s*\(\s*echo\s*,\s*0\s*\)\s*(;\s*)?$//ms) {        # Try to matchin in $rest:
+          if ($rest =~ s/\A(.*?)^default\s*\(\s*echo\s*,\s*(?!0\b)\S[^\n]*\)\s*(;\s*)?$//ms) {
+	    (my $Pref = my $cmd = "$1") =~ s/^(.*)$/### $1/mg ;
+	    if ($cmd =~ /(?:^|;)\s*(?:print|error)\(/ or $userfun and $cmd =~ / \b ($userfun) \s* \( /x) {
+	      my $with = '';
+	      if ($cmd =~ /^\s*(\w+)\s*\($matched_par\)\s*=(?!=)/) {
+	        $with = "with a user function $1() ";
+		if (defined $userfun) {
+		  $userfun .= "|$1";
+		} else {
+		  $userfun = $1;
+		}
+	      }
+	      print "${Pref}ok $current_num # Skipping (no-echo group ${with}containing print() or error() or a converted-to-Perl variable)\n";
+	    } else {
+              print("# The following `? ' are the output from parse_as_gp() for\n$Pref");
+              my $Err;
+              eval { parse_as_gp $cmd, sub ($) {"main::".CORE::shift}, 'echo'; 1}
+                or $Err = $@;
+              # next		# fail to process, but do emit the expected-for-test output
+              if (defined $Err) {
+                print "# `$in'\nnot ok $current_num # in no-echo group err=$Err\n";
+              } else {
+                print "# `$in'\nok $current_num # Skipping translation to Math::Pari (no-echo group done in PARI, see above)\n";
+              }
+	    }
+	    $RET = 1;
+          } else {
+            warn("Cannot find a matching echo-on command in input file"),
+          }
+        } else {
+          warn("Cannot find a matching echo-off command in input file"),
+        }
+      } else {
+        # It is not clear why changevar gives a different answer in GP
+        print "# `$in'\nok $current_num # Skipping (converting test for '$1' needs additional work)\n";
+        $RET = 1;
+      }
     } elsif ($userfun
 	     and $in =~ / \b ($userfun) \s* \( /x) {
-      print "# `$in'\nok $current_num # Skipping (user function)\n";
-      return;
+      print "# `$in'\nok $current_num # Skipping (user function containing print() or error() or a converted-to-Perl variable)\n";
+      $RET = 1;
     } elsif ($installed
 	     and $in =~ / \b ($installed) \s* \( /x) {
       print "# `$in'\nok $current_num # Skipping (installed function)\n";
-      return;
+      $RET = 1;
 #    } elsif ($in =~ / \b ( sizebyte ) \b /x
 #	     and $file !~ /will_fail/) {
 #      # XXXX Will result in a wrong answer, but we moved these tests to a different
 #      print "# `$in'\nok $current_num # Skipping (would fail, checked in different place)\n";
-#      return;
+#      $RET = 1;
     } elsif ($in =~ /\b(nonesuch now|nfisincl)\b/) {
       print "# `$in'\nok $current_num # Skipping (possibly FATAL $1)\n";
-      return;
+      $RET = 1;
+    } elsif ($in =~ $skip_fun_rx) {
+      print "# `$in'\nok $current_num # Skipping (see a PARI function $1() calling print() or error())\n";
+      $RET = 1;
     }
     # Convert transposition
     $in =~ s/(\$?\w+(\([^()]*\))?|\[([^\[\]]+(?=[\[\]])|\[[^\[\]]*\])*\])~/mattranspose($1)/g;
     if ($in =~ /~/) {
       print "# `$in'\nok $current_num # Skipping (transpose notation)\n";
-      return;
+      $RET = 1;
     }
+    if ($in =~ /->/) {
+      print "# `$in'\nok $current_num # Skipping (-> notation)\n";
+      $RET = 1;
+    }
+    if ($RET and $in =~ /^(\w+)\([^()]*\)=(?!=)/) {
+      my $n = $1;
+      if ($in =~ /\b(print|error|$ourvars_rx)\b/) {
+	push @skip_fun, $n;
+	my $rx = join '|', @skip_fun;
+	$skip_fun_rx = qr/\b($rx)\(/;
+	if ($1 eq 'print') {
+	  print "# NOT doing PARI-eval: print() seen\n";
+	} else {
+	  print "# NOT doing PARI-eval: variable `$1' converted to Perl before was seen\n";
+	}
+	if (defined $userfun) {
+	  $userfun .= "|$n";
+	} else {
+	  $userfun = $n;
+	}
+	return;
+      }
+      print "# doing PARI-eval of a function `$n' instead\n";
+      PARI $IN;
+	no strict 'refs';
+      *$n = Math::Pari::__wrap_PARI_macro $n;
+    }
+    return if $RET;
     if ($in =~ /^\s*alias\s*\(\s*(\w+)\s*,\s*(\w+)\s*\)$/) {
       print "# Aliasing `$1' ==> `$2'\nok $current_num\n";
       no strict 'refs';
@@ -427,24 +543,24 @@ sub process_test {
     if ($in !~ /\w\(/) { # No function calls
       # XXXX Primitive!
       # Constants: (.?) eats following + or - (2+3 ==> PARI(2)+PARI(3))
-      $in =~ s/(^|\G|\W)([-+]?)(\d+(\.\d*)?)(.?)/$1 $2 PARI($3) $5/g;
+      $in =~ s/(^|\G|\W)([-+]?)(\d+(\.\d*)?(?:e-?\d+)?)(.?)/$1 $2 PARI($3) $5/gi;
       # Big integer constants:
       $in =~ s/\bPARI\((\d{10,})\)/PARI('$1')/g;
     } elsif ($in =~ /\b(elllseries|binomial|mathilbert|intnum|intfuncinit|intfuncinit)\b/) { # high precision needed?
       # XXXX Primitive!
       # Substitute constants where they are not arguments to functions,
       # (except small half-integers, which should survive conversion)
-      $in =~ s/(^|\G|\W)([-+]?)(?!\d{0,6}\.5\b)(\d+\.\d*)/$1 $2 PARI('$3') /g;
+      $in =~ s/(^|\G|\W)([-+]?)(?!\d{0,6}\.5\b)(\d+\.\d*(?:e-?\d+)?)/$1 $2 PARI('$3') /gi;
       # Bug in GP???  Too sensitive to precision of 4.5 in intfuncinit(t=[-oo, 4.5],[oo, 4.5], gamma(2+I*t)^3, 1);
-      $in =~ s/(^|\G|\W)([-+]?)(\d+\.\d*)/$1 $2 PARI('$3') /g
+      $in =~ s/(^|\G|\W)([-+]?)(\d+\.\d*(?:e-?\d+)?)/$1 $2 PARI('$3') /gi
 	if $in =~ /intfuncinit/;
       # Big integer constants:
       $in =~ s/\bPARI\((\d{10,})\)/PARI('$1')/g;
     } else {
       # Substitute big integer constants
-      $in =~ s/(^|\G|\W)(\d{10,}(?!\.\d*))(.?)/$1 PARI('$2') $3/g;
+      $in =~ s/(^|\G|\W)(\d{10,}(?!\.\d*(?:e-?\d+)?))(.?)/$1 PARI('$2') $3/gi;
       # Substitute division
-	$in =~ s/(^|[\-\(,\[])(\d+)\s*\/\s*(\d+)(?=$|[\),\]])/$1 PARI($2)\/PARI($3) /g;
+	$in =~ s/(^|[\-\(,\[=+*\/])(\d+)\s*\/\s*(\d+)(?=$|[*+\-\/\),\]])/$1 PARI($2)\/PARI($3) /g;
     }
     my %seen_now;
     # Substitute i= in loop commands
@@ -454,6 +570,9 @@ sub process_test {
     # Substitute print
     $in =~ s/\b(|p|tex)print(tex|)\(/ 'my_' . $1 . $2 . 'print(1,' /ge;
     $in =~ s/\b(|p|tex)print1\(/ 'my_' . $1 . 'print(0,'/ge;
+    if ($skip_eval and $in =~ /\beval\(/g) { # eval($y)
+      print("# `$in'\nok $current_num # Skipping: eval's signature C is not supported yet\n"), return;
+    }
     $in =~ s/\b(eval|shift|sort)\(/&$1\(/g; # eval($y)
     # Special case -oo (with $oo=[PARI(1)] done earlier;
     # Having $oo defined without external PARI tests conversions; but it'sn't overloaded in older PARI
@@ -462,11 +581,14 @@ sub process_test {
     # s/\b(direuler\s*\(\s*\w+\s*),/$1=/;	# direuler
     $in =~ s/\bX\b/PARIvar("X")/g if $in =~ /\bdireuler\b/;
     $in =~ s/(^\s*|[;(]\s*)(?=(\w+)\s*=\s*)/$seen_now{$2} = '$'; pre_update_seen($2); $1 . '$'/ge; # Assignment
+    if ($in =~ /\.((?!\d|(?<=\d\.)e-?\d)\w+(?![\w"]))/) {
+      print("# `$in'\nok $current_num # Skipping: methods not supported yet (.$1)\n"), return;
+    }
     # Prepend $ to variables (not before '^' - inside of 'o(x^17)'):
     $in =~ s/(^|[^\$])\b([a-zA-Z]\w*)\b(?!\s*[\(^])/
       		($1 || '') . ($seen{$2} || $seen_now{$2} || '') . $2
 	/ge;
-    # Die if did not substitute variables:
+    # Skip if did not substitute variables:
     while ($in =~ /(^|[^\$])\b([a-zA-Z]\w*)\b(?!\s*[\{\(^])/g) {
       print("# `$in'\nok $current_num # Skipping: variable `$2' was not set\n"), return
 	unless $seen{$2} and $seen{$2} eq ' ' or $in =~ /\"/;
@@ -538,7 +660,7 @@ sub process_test {
 		     )*		# One level of func-call (PARI('.3')) supported
 		  [,=]){2}		# $x,1
 		)				# end group 1
-		(?!\s*sub\s*\{)	# Skip already converted...
+		(?!\s*sub(?:\s*\(\$*\))?\s*\{)	# Skip already converted...
 		(		# 2: This follows after a comma on toplevel
 		  (?:
 		    [^(,)\[\]]+
@@ -578,7 +700,7 @@ sub process_test {
 		  )*		# Two levels of parenths supported
 		)				# end group 2
 		(?= [),] )
-	      /$1 sub{$2}/xg;
+	      /subify_iterators("$1","$2")/xge;
     # Do the rest (do not take = after the variable name)
     1 while
       $in =~ s/
@@ -590,7 +712,7 @@ sub process_test {
 		    (?:
 		      post
 		    )?
-		    ploth (?! raw \b ) \w+
+		    ploth (?! raw (?:export)? \b ) \w+
 		  |
 		    # sum \w*
 		    sum (?! alt | num (?:init)? \b) \w+
@@ -621,7 +743,7 @@ sub process_test {
 		  )+		# One level of parenths supported
 		  ,
 		)
-		(?!\s*sub\s*\{)	# Skip already converted...
+		(?!\s*sub(?:\s*\(\$*\))?\s*\{)	# Skip already converted...
 		(		# 3: This follows after a comma on toplevel
 		  (?:
 		    [^(,)\[\]]+
@@ -653,7 +775,7 @@ sub process_test {
 		  )*		# Two levels of parenths supported
 		)
 		\)
-	      /$1 sub{$3}\)/xg;
+	      /subify_iterators("$1","$3") . ')'/xge;
     # Convert 10*20 to integer
     $in =~ s/(\d+)(?=\*\*\d)/ PARI($1) /g;
     # Convert blah[3], blah()[3] to blah->[-1+3]
@@ -675,6 +797,12 @@ sub process_test {
 		       or $in =~ /\b( ellinit|zeta|bin|comprealraw|frac|
 				      lseriesell|powrealraw|legendre|suminf|
 				      forstep )\b/x);
+    my $newvars;
+    $ourvars{$1}++ or $newvars++ while $in =~ /(?<!my )\$(\w+)\s*=(?!=)/g;
+    if ($newvars) {
+      $ourvars_rx = join '|', sort keys %ourvars;
+      $ourvars_rx = qr($ourvars_rx);
+    }
     # Remove the value from texprint:
     # pop @$out if $in =~ /texprint/ and @$out == 2;
     my $pre_time = time() - $ini_time;
@@ -709,6 +837,7 @@ sub process_test {
 	$rout = mformat @$out;
 	if (defined $rres and $rres !~ /\n/) {
 	  $rout =~ s/\]\s*\[/; /g;
+	  $rout =~ s/\[\s+/[/g;
 	  $rout =~ s/,\n/, \n/g; # Spaces were removed
 	  $rout =~ s/\n//g;	# Long wrapped text
 	}
@@ -762,12 +891,17 @@ sub process_test {
     }
     my $cmp;
     if (defined $rres and defined $re_out) {
-      $cmp = eval { $rres =~ /^$re_out$/ };
-      if ($@ and $@ =~ /regexp too big/) {
-	print "ok $current_num # Skipped: $@\n";
-	update_seen \%seen_now;
-	$prev = $res;
-	return;
+      for my $how (0,'Mat') {
+        my $RR = $rres;
+        $RR =~ s/\bMat\(($matched_par)\)/[$1]/g if $how;
+        $cmp = eval { $RR =~ /^$re_out$/ };
+        if ($@ and $@ =~ /regexp too big/) {
+          print "ok $current_num # Skipped: $@\n";
+          update_seen \%seen_now;
+          $prev = $res;
+          return;
+        }
+        last if $cmp
       }
     }
     my $post_time = time() - $ini_time - $pre_time - $run_time;
@@ -780,7 +914,7 @@ sub process_test {
       print "ok $current_num #  run_time=$run_time, post_time=$post_time, pre_time=$pre_time\n";
       update_seen \%seen_now;
       $prev = $res;
-    } elsif (not $noans and (not defined $rres or $rres ne $rout)) {
+    } elsif (not $noans and (not defined $rres or filter_res $rres ne filter_res $rout)) {
       nok_print $current_num, $in, "not ok $current_num # in='$in'\n#    out='", $rres, "', type='", ref $res,
       "'\n# expect='$rout'\n";
     } elsif ($doprint and $printout ne $rout) {
@@ -797,7 +931,7 @@ sub process_test {
 sub process_error {
   my ($in, $out, $error) = @_;
   $current_num++;
-  print("# `$in'\nok $current_num # Skipping: test producing error unsupported yet ($error)\n");
+  print("# `$in'\nok $current_num # Skipping: test producing PARI error unsupported yet (err=$error)\n");
 }
 
 sub process_definition {
@@ -861,6 +995,12 @@ sub prec {
   print "# Setting precision to $_[0] digits.\n";
   print "ok $current_num\n" unless $_[1];
 }
+sub bprec {
+  PARI("default(realbitprecision,$_[0])");		# setbitprecision($_[0]);
+  print "# Setting bitprecision to $_[0] bits.\n";
+  print "# bitprecision: res=", PARI("default(realbitprecision)"),"\n";
+  print "ok $current_num\n" unless $_[1];
+}
 sub sprec {
   setseriesprecision($_[0]);
   print "# Setting series precision to $_[0] digits.\n";
